"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var BindRestContext_1;
Object.defineProperty(exports, "__esModule", { value: true });
const url = __importStar(require("url"));
const QueryString = __importStar(require("querystring"));
const cookie = __importStar(require("cookie"));
const bind_di_1 = require("bind-di");
const lowercase_keys_1 = __importDefault(require("lowercase-keys"));
const debug = require('debug')('bind:rest:context');
const TAG = 'ContextClass';
let BindRestContext = BindRestContext_1 = class BindRestContext {
    constructor() {
        this.contextType = 'BindRestContext';
        this.responseCookies = {};
        this.myControllerName = '';
        this.responseHeaders = {};
        this.requestStartTime = 0;
        this.scopedComponents = [];
        this.storage = {};
        this.errors = [];
        this.scope = bind_di_1.ComponentScope.REQUEST;
        this.controllerArguments = [];
    }
    setHeader(key, value) {
        this.responseHeaders[key] = value;
    }
    setStatusCode(statusCode) {
        this.responseStatusCode = statusCode;
    }
    get req() {
        return this.httpRequest;
    }
    set req(req) {
        this.httpRequest = req;
        this.reqUrl = req.url;
        this.httpRequestMethod = req.method.toUpperCase();
        this.requestStartTime = Date.now();
    }
    get requestHeaders() {
        return this.req.headers;
    }
    get requestBody() {
        return this.httpRequestBody;
    }
    set requestBody(body) {
        this.httpRequestBody = body;
    }
    get requestMethod() {
        return this.httpRequestMethod;
    }
    set appResponse(response) {
        this.response = response;
    }
    get appResponse() {
        debug('%s enter get appResponse()', TAG);
        if (!this.response) {
            return undefined;
        }
        if (this.responseStatusCode) {
            this.response.statusCode = this.responseStatusCode;
        }
        this.response.headers = Object.assign(Object.assign({}, lowercase_keys_1.default(this.responseHeaders)), this.response.headers);
        if (this.response.cookies) {
            this.response.cookies = Object.assign(Object.assign({}, this.response.cookies), this.responseCookies);
        }
        else {
            this.response.cookies = this.responseCookies;
        }
        return this.response;
    }
    get startTime() {
        return this.requestStartTime;
    }
    getComponent(id) {
        if (bind_di_1.isSameIdentity(id, BindRestContext_1.id)) {
            debug('%s getComponent Returning instance of self', TAG);
            return this;
        }
        const res = this.scopedComponents.find((component) => bind_di_1.isSameIdentity(component.identity, id));
        return res === null || res === void 0 ? void 0 : res.component;
    }
    setComponent(identity, component) {
        if (!bind_di_1.isSameIdentity(identity, BindRestContext_1.id)) {
            this.scopedComponents.push({ identity, component });
        }
    }
    get controllerName() {
        return this.myControllerName;
    }
    get routeParams() {
        return this.myRouteParams;
    }
    set routeParams(params) {
        if (!this.myRouteParams) {
            this.myRouteParams = params;
        }
        else {
            throw new Error('routeParams is already set Cannot set new value');
        }
    }
    set controllerName(name) {
        if (!this.myControllerName) {
            this.myControllerName = name;
        }
        else {
            throw new Error(`Controller name is already set to '${this.myControllerName}' Cannot set new value '${name}'`);
        }
    }
    get path() {
        return this.parseUrl().pathname;
    }
    get requestUrl() {
        return this.reqUrl;
    }
    parseUrl() {
        if (!this.uriInfo) {
            this.uriInfo = url.parse(this.reqUrl);
        }
        return this.uriInfo;
    }
    get querystring() {
        const uri = this.parseUrl();
        return (uri === null || uri === void 0 ? void 0 : uri.query) || '';
    }
    get parsedUrlQuery() {
        if (!this.query) {
            this.query = QueryString.parse(this.querystring);
        }
        return this.query;
    }
    get parsedCookies() {
        if (this.cookies)
            return this.cookies;
        if (this.requestHeaders.cookie) {
            try {
                this.cookies = cookie.parse(this.requestHeaders.cookie);
            }
            catch (e) {
                debug('% Failed to parse cooke header %s', TAG, e.mesage);
                this.cookies = {};
            }
        }
        else {
            debug('%s NO cookie header in request');
            this.cookies = {};
        }
        return this.cookies;
    }
    getResponseCookies() {
        return this.responseCookies;
    }
    setResponseCookie(name, value) {
        this.responseCookies[name] = value;
    }
};
BindRestContext.id = bind_di_1.Identity(BindRestContext_1);
BindRestContext.create = (req) => {
    const instance = new BindRestContext_1();
    instance.req = req;
    return instance;
};
BindRestContext = BindRestContext_1 = __decorate([
    bind_di_1.Component,
    bind_di_1.Scope(bind_di_1.ComponentScope.REQUEST)
], BindRestContext);
exports.default = BindRestContext;
//# sourceMappingURL=context.js.map