"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RouterMiddleware = exports.HttpRouter = void 0;
const holiday_router_1 = require("holiday-router");
Object.defineProperty(exports, "HttpRouter", { enumerable: true, get: function () { return holiday_router_1.HttpRouter; } });
const bind_di_1 = require("bind-di");
const decorators_1 = require("../lib/decorators");
const consts_1 = require("../lib/consts");
const utils_1 = require("../lib/utils");
const errors_1 = require("../lib/errors");
const debug = require('debug')('bind:rest:runtime');
const TAG = 'RouterMiddleware';
bind_di_1.Component(holiday_router_1.HttpRouter);
bind_di_1.Singleton(holiday_router_1.HttpRouter);
let RouterMiddleware = class RouterMiddleware {
    doFilter(context) {
        const { requestMethod } = context;
        const { path } = context;
        debug('%s entered filter with contextType=%s method="%s" url="%s"', TAG, context.contextType, requestMethod, context.path);
        const httpMethod = utils_1.toHTTPMethod(requestMethod);
        debug('%s httpMethod="%s"', TAG, httpMethod);
        debug('%s path="%o"', TAG, path);
        const routeMatch = this.router.getRouteMatch(httpMethod, path);
        if (!routeMatch) {
            debug('%s NO match for method="%s" url="%s"', TAG, requestMethod, path);
            return Promise.reject(new errors_1.NotFoundError(`
      Resource not found for method="${requestMethod}" 
      url="${context.requestUrl}"`));
        }
        const ctrlContainer = routeMatch.node.controllers.find((ctrl) => {
            return ctrl.matcher(context);
        });
        if (!ctrlContainer) {
            debug('%s No matching ControllerContainer found for method="%s" url="%s"', TAG, requestMethod, context.requestUrl);
            return Promise.reject(new errors_1.NotFoundError(`Controller not found for this request`));
        }
        Reflect.set(context, 'routeParams', routeMatch === null || routeMatch === void 0 ? void 0 : routeMatch.params);
        return ctrlContainer.controller(context).then((response) => {
            debug('%s got appResponse %o', TAG, response);
            Reflect.set(context, 'appResponse', response);
            return context;
        });
    }
};
__decorate([
    bind_di_1.Inject,
    __metadata("design:type", holiday_router_1.HttpRouter)
], RouterMiddleware.prototype, "router", void 0);
RouterMiddleware = __decorate([
    bind_di_1.Singleton
], RouterMiddleware);
exports.RouterMiddleware = RouterMiddleware;
decorators_1.decorateMiddleware(RouterMiddleware, consts_1.PRIORITY_ROUTER, 'RouterMiddleware');
//# sourceMappingURL=router.js.map