"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Application = exports.validateOptions = void 0;
const path_1 = __importDefault(require("path"));
const bind_di_1 = require("bind-di");
const context_1 = __importDefault(require("../../components/context"));
const setuproutes_1 = __importDefault(require("./apputils/setuproutes"));
const getmiddlewares_1 = __importDefault(require("./apputils/getmiddlewares"));
const defaulterrorformatter_1 = require("./apputils/defaulterrorformatter");
const processexithelper_1 = __importDefault(require("../utils/processexithelper"));
const applicationerror_1 = __importDefault(require("../errors/applicationerror"));
const getresponsefromcontext_1 = require("./apputils/getresponsefromcontext");
const defaultresponsewriter_1 = require("./apputils/defaultresponsewriter");
const debug = require('debug')('bind:rest:runtime:application');
const TAG = 'APPLICATION';
const APPLICATION_COMPONENTS_DIR = path_1.default.resolve(__dirname, '../../components');
const validateOptions = (options) => {
    if (options.baseUrl) {
        if (!options.baseUrl.startsWith('/')) {
            throw new Error(`Bad value of baseUrl option "${options.baseUrl}" baseUrl must start with a "/"`);
        }
        if (options.baseUrl.endsWith('/')) {
            throw new Error(`Bad value of baseUrl option "${options.baseUrl}" baseUrl cannot end with a "/"`);
        }
    }
};
exports.validateOptions = validateOptions;
class Application {
    constructor(options) {
        this.middlewares = [];
        this.settings = options;
        this.bindContainer = new bind_di_1.Container();
        processexithelper_1.default(this);
    }
    get settings() {
        return this.configOptions;
    }
    set settings(options) {
        exports.validateOptions(options);
        this.configOptions = options;
    }
    get container() {
        return this.bindContainer;
    }
    setContainer(container) {
        return __awaiter(this, void 0, void 0, function* () {
            bind_di_1.load(container, [...this.configOptions.componentDirs, APPLICATION_COMPONENTS_DIR], this.settings.envOverrideVar);
            if (this.settings.extraComponents) {
                debug('%s have %s extraComponents to set', TAG, this.settings.extraComponents.length);
                try {
                    this.settings.extraComponents.forEach((component) => bind_di_1.addComponent(container, component));
                }
                catch (e) {
                    throw new applicationerror_1.default(`Failed to add extra component ${e.message}`, e);
                }
            }
            try {
                yield container.initialize();
                setuproutes_1.default(container);
                this.middlewares = getmiddlewares_1.default(container);
                debug('%s got %d middleware functions', TAG, this.middlewares.length);
                this.defaultResponseWriter = defaultresponsewriter_1.getResponseWriter(container);
                this.defaultErrorFormatter = defaulterrorformatter_1.getErrorFormatter(container);
                this.bindContainer = container;
                return this.bindContainer;
            }
            catch (e) {
                throw new applicationerror_1.default(`Failed to initialize container Error=${e.message}`, e);
            }
        });
    }
    onExit(exitCode) {
        debug('%s %s onExit called with code=%d', TAG, this.toString(), exitCode);
        return this.container.cleanup().then((status) => {
            debug('%s Container cleanup completed with res="%s"', TAG, status);
            return exitCode;
        });
    }
    getErrorFormatter(context) {
        return this.defaultErrorFormatter;
    }
    getResponseWriter(context) {
        return this.defaultResponseWriter;
    }
    getAppResponse(context) {
        const handlerPromise = this.middlewares.reduce((prev, next) => {
            return prev.then(next);
        }, Promise.resolve(context));
        const runners = [handlerPromise];
        return Promise.race(runners)
            .then(getresponsefromcontext_1.getResponseFromContext)
            .catch((error) => {
            const errorFormatter = this.getErrorFormatter(context);
            const appError = errorFormatter(error);
            return {
                statusCode: appError.statusCode,
                headers: getresponsefromcontext_1.filterStringOnlyHeaders(appError.headers),
                body: appError.body,
                getReadStream: appError.getReadStream.bind(appError),
            };
        });
    }
    handleRequest(req, res) {
        const ctx = context_1.default.create(req);
        this.getAppResponse(ctx).then((serverResponse) => this.getResponseWriter(ctx)(serverResponse, res));
    }
    init() {
        return this.setContainer(this.container).then((previous) => {
            debug('%s finished setContainer. has previousContainer %s', TAG, bind_di_1.isDefined(previous));
            return true;
        });
    }
    toString() {
        var _a, _b, _c;
        return `Application Instance with ${(_c = (_b = (_a = this.bindContainer) === null || _a === void 0 ? void 0 : _a.components) === null || _b === void 0 ? void 0 : _b.length) !== null && _c !== void 0 ? _c : 0} components`;
    }
}
exports.Application = Application;
//# sourceMappingURL=application.js.map