"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getResponseWriter = exports.defaultResponseWriter = void 0;
const headernames_1 = __importDefault(require("../../consts/headernames"));
const debug = require('debug')('bind:rest:runtime:responsewriter');
const TAG = 'DEFAULT_RESPONSE_WRITER';
const defaultResponseWriter = (serverResponse, res) => {
    if (res.writableFinished) {
        debug('%s response already sent. Skipping responseWriter middleware', TAG);
        return undefined;
    }
    res.on('finished', () => debug('%s res onFinished', TAG));
    res.on('end', () => debug('%s res onEnd called', TAG));
    res.on('close', () => debug('%s res onClosed', TAG));
    res.on('error', (error) => debug('%s res onError error=%o', TAG, error));
    if (serverResponse.statusCode) {
        res.statusCode = serverResponse.statusCode;
    }
    if (res.headersSent) {
        debug('%s headers already sent.', TAG);
    }
    else {
        if (serverResponse.headers) {
            for (const [key, value] of Object.entries(serverResponse.headers)) {
                try {
                    res.setHeader(key, value);
                }
                catch (e) {
                    debug('%s error setting response header "%s" error="%s"', TAG, key, value);
                }
            }
        }
        if (serverResponse.cookies && serverResponse.cookies.length > 0) {
            debug('%s serverResponse has %s cookie ', TAG, serverResponse.cookies.length);
            serverResponse.cookies.forEach((cookie) => {
                debug('%s Setting set-cookie with %s', TAG, cookie);
                try {
                    res.setHeader(headernames_1.default.SET_COOKIE, cookie);
                }
                catch (e) {
                    debug('%s error setting cookie "%s" error="%s"', TAG, cookie, e);
                }
            });
        }
        else {
            debug('%s serverResponse does not have cookies', TAG);
        }
    }
    if (serverResponse.body) {
        debug('%s serverResponse has body', TAG);
        res.end(serverResponse.body);
    }
    else {
        serverResponse.getReadStream().pipe(res);
    }
    return undefined;
};
exports.defaultResponseWriter = defaultResponseWriter;
const getResponseWriter = (cntr) => {
    debug('%s entered getResponseWriter. %s', TAG, cntr);
    return defaultResponseWriter;
};
exports.getResponseWriter = getResponseWriter;
//# sourceMappingURL=defaultresponsewriter.js.map