"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bind_di_1 = require("bind-di");
const metaprops_1 = require("../../decorators/metaprops");
const applicationerror_1 = __importDefault(require("../../errors/applicationerror"));
const debug = require('debug')('bind:rest:runtime:middleware');
const TAG = 'MIDDLEWARE';
function getMiddlewares(ctr) {
    const ret = ctr.components.filter((c) => {
        var _a;
        return ((_a = c.componentMetaData) === null || _a === void 0 ? void 0 : _a[metaprops_1.MIDDLEWARE_PRIORITY]) !== undefined;
    });
    if (!bind_di_1.isDefined(ret) || ret.length === 0) {
        throw new applicationerror_1.default(`No Middleware components found in container.
    Container should have at least the RouterMiddleware which is part of this framework.
    Looks like RouterMiddleware was not loaded`);
    }
    ret.sort((a, b) => {
        return a.componentMetaData[metaprops_1.MIDDLEWARE_PRIORITY] - b.componentMetaData[metaprops_1.MIDDLEWARE_PRIORITY];
    });
    const aMW = ret.map((comp) => {
        const mwName = bind_di_1.getComponentNameFromIdentity(comp.identity);
        const priority = comp.componentMetaData[metaprops_1.MIDDLEWARE_PRIORITY];
        const func = (ctx) => {
            const mw = comp.get([ctx]);
            debug('%s applying middleware mw="%s" priority="%s"', TAG, mwName, priority);
            return mw.doFilter(ctx);
        };
        func[metaprops_1.SYM_MIDDLEWARE_NAME] = mwName;
        func[metaprops_1.SYM_MIDDLEWARE_PRIORITY] = priority;
        return func;
    });
    return aMW;
}
exports.default = getMiddlewares;
//# sourceMappingURL=getmiddlewares.js.map