"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.filterStringOnlyHeaders = exports.getResponseFromContext = exports.toStringCookies = void 0;
const cookies_1 = __importDefault(require("cookies"));
const interfaces_1 = require("../../interfaces");
const appresponse_1 = require("../appresponse");
const consts_1 = require("../../consts");
const bytelength_1 = __importDefault(require("../../utils/bytelength"));
const applicationerror_1 = __importDefault(require("../../errors/applicationerror"));
const debug = require('debug')('bind:rest:runtime:application');
const TAG = 'GET_RESPONSE_FROM_CONTEXT';
const toStringCookies = (appResponse) => {
    debug('%s entered toStringCookies', TAG);
    const { cookies } = appResponse;
    const { headers } = appResponse;
    let ret;
    if (cookies) {
        debug('%s appResponse has cookies', TAG);
        ret = Object.entries(cookies).map(([key, val]) => {
            return new cookies_1.default.Cookie(key, val.value, val.options).toHeader();
        });
    }
    if (headers && headers[consts_1.HEADER_NAMES.SET_COOKIE]) {
        debug('%s appResponse has own set-cookie header', TAG);
        ret = ret || [];
        ret = ret.concat(headers[consts_1.HEADER_NAMES.SET_COOKIE]);
    }
    debug('%s returning from toStringCookies ret=%s', TAG, ret);
    return ret;
};
exports.toStringCookies = toStringCookies;
const filterStringOnlyHeaders = (headers) => {
    const entries = Object.entries(headers);
    const ret = entries.reduce((acc, next) => {
        const [key, val] = next;
        if (!Array.isArray(val)) {
            acc[key] = `${val};`;
        }
        else {
            debug('%s Removing header % because its a multi-value', TAG, key);
        }
        return acc;
    }, {});
    return ret;
};
exports.filterStringOnlyHeaders = filterStringOnlyHeaders;
const getResponseFromContext = (context) => {
    debug('%s entered getResponseFromContext', TAG);
    const response = context.appResponse ||
        new appresponse_1.ErrorResponse(500, 'Response not processed');
    let responseCookies;
    try {
        responseCookies = toStringCookies(response);
    }
    catch (e) {
        throw new applicationerror_1.default('Error processing response cookies', e);
    }
    const ret = {
        statusCode: response.statusCode,
        headers: filterStringOnlyHeaders(response.headers),
        cookies: responseCookies,
        getReadStream: response.getReadStream.bind(response),
    };
    if (interfaces_1.isAppResponseWithBody(response)) {
        debug('%s response has body', TAG);
        ret.body = response.body;
        ret.headers[consts_1.HEADER_NAMES.CONTENT_LENGTH] = `${bytelength_1.default(ret.body)}`;
    }
    return ret;
};
exports.getResponseFromContext = getResponseFromContext;
//# sourceMappingURL=getresponsefromcontext.js.map