"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bind_di_1 = require("bind-di");
const holiday_router_1 = require("holiday-router");
const getcontrollercomponents_1 = __importDefault(require("../../utils/getcontrollercomponents"));
const parsecontroller_1 = __importDefault(require("../parsecontroller"));
const applicationerror_1 = __importDefault(require("../../errors/applicationerror"));
const frameworkcontroller_1 = __importDefault(require("../frameworkcontroller"));
const debug = require('debug')('bind:rest:context');
const TAG = 'SetupRoutes';
function setupRoutes(container) {
    const controllerComponents = getcontrollercomponents_1.default(container);
    debug('%s Found %d controller components', TAG, controllerComponents.length);
    const parsedControllers = controllerComponents
        .map((c) => {
        return parsecontroller_1.default(container)(c);
    })
        .flat();
    if (!bind_di_1.arrayNotEmpty(parsedControllers)) {
        throw new applicationerror_1.default('No Controller components found');
    }
    const router = container.getComponent(bind_di_1.Identity(holiday_router_1.HttpRouter));
    parsedControllers.forEach((controllerDetails) => {
        controllerDetails.requestMethods.forEach((method) => {
            router.addRoute(method, controllerDetails.routePath, new frameworkcontroller_1.default(controllerDetails));
        });
    });
}
exports.default = setupRoutes;
//# sourceMappingURL=setuproutes.js.map