"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContextParam = exports.CookieParam = exports.HeaderParam = exports.QueryParam = exports.PathParam = exports.delegateParamAnnotation = exports.doParamAnnotation = void 0;
require("reflect-metadata");
const bind_di_1 = require("bind-di");
const metaprops_1 = require("../metaprops");
const controllerparamtype_1 = __importDefault(require("../../enums/controllerparamtype"));
const getparamtype_1 = __importDefault(require("./getparamtype"));
const makeparamextractorfactory_1 = __importDefault(require("./makeparamextractorfactory"));
const debug = require('debug')('bind:rest:decorators');
const TAG = 'METHOD-ARGUMENTS';
function applyParamAnnotation(methodArgumentDetail, target, propertyKey) {
    const index = methodArgumentDetail.position;
    const targetStereoType = bind_di_1.getTargetStereotype(target);
    debug('%s targetStereoType="%s"', TAG, targetStereoType);
    if (targetStereoType !== bind_di_1.TargetStereoType.PROTOTYPE) {
        throw new TypeError(`${controllerparamtype_1.default[methodArgumentDetail.type]} 
    can be added only to class method`);
    }
    debug(`Defining @PathParam ${String(methodArgumentDetail.name)} for arg ${index} 
  of method ${target.constructor.name}.${String(propertyKey)}`);
    const metaDetails = Reflect.getMetadata(metaprops_1.SYM_METHOD_PARAMS, target, propertyKey) || [];
    const paramTypes = Reflect.getMetadata(bind_di_1.PARAM_TYPES, target, propertyKey);
    if (metaDetails[index]) {
        if (metaDetails[index].f && methodArgumentDetail.type) {
            throw new Error(`Method parameter ${index} already defined 
      on method ${target.constructor.name}.${String(propertyKey)} 
      - ${JSON.stringify(metaDetails[index])}`);
        }
        metaDetails[index].f = makeparamextractorfactory_1.default(methodArgumentDetail.type, methodArgumentDetail.name);
        metaDetails[index].paramName = methodArgumentDetail.name;
        metaDetails[index].paramDecoratorType = methodArgumentDetail.type;
        metaDetails[index].paramType = getparamtype_1.default(paramTypes, index);
    }
    else {
        metaDetails[index] = {
            f: makeparamextractorfactory_1.default(methodArgumentDetail.type, methodArgumentDetail.name),
            paramDecoratorType: methodArgumentDetail.type,
            isRequired: false,
            paramName: methodArgumentDetail.name,
            paramType: getparamtype_1.default(paramTypes, index),
        };
    }
    Reflect.defineMetadata(metaprops_1.SYM_METHOD_PARAMS, metaDetails, target, propertyKey);
    return undefined;
}
const doParamAnnotation = (name, paramType) => (target, propertyKey, parameterIndex) => {
    return applyParamAnnotation({
        type: paramType,
        name,
        position: parameterIndex,
    }, target, propertyKey);
};
exports.doParamAnnotation = doParamAnnotation;
const delegateParamAnnotation = (target) => (paramType) => (propertyKey, parameterIndex) => {
    if (typeof target === 'string') {
        return exports.doParamAnnotation(target, paramType);
    }
    const paramName = bind_di_1.getMethodParamName(target, propertyKey, parameterIndex);
    exports.doParamAnnotation(paramName, paramType)(target, propertyKey, parameterIndex);
    return undefined;
};
exports.delegateParamAnnotation = delegateParamAnnotation;
function PathParam(target, propertyKey, parameterIndex) {
    return exports.delegateParamAnnotation(target)(controllerparamtype_1.default.PathParam)(propertyKey, parameterIndex);
}
exports.PathParam = PathParam;
function QueryParam(target, propertyKey, parameterIndex) {
    return exports.delegateParamAnnotation(target)(controllerparamtype_1.default.QueryParam)(propertyKey, parameterIndex);
}
exports.QueryParam = QueryParam;
function HeaderParam(target, propertyKey, parameterIndex) {
    return exports.delegateParamAnnotation(target)(controllerparamtype_1.default.Header)(propertyKey, parameterIndex);
}
exports.HeaderParam = HeaderParam;
function CookieParam(target, propertyKey, parameterIndex) {
    return exports.delegateParamAnnotation(target)(controllerparamtype_1.default.Cookie)(propertyKey, parameterIndex);
}
exports.CookieParam = CookieParam;
function ContextParam(target, propertyKey, parameterIndex) {
    return exports.delegateParamAnnotation(target)(controllerparamtype_1.default.ContextParam)(propertyKey, parameterIndex);
}
exports.ContextParam = ContextParam;
//# sourceMappingURL=complexmethodparams.js.map