"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const controllerparamtype_1 = __importDefault(require("../../enums/controllerparamtype"));
const systemerror_1 = __importDefault(require("../../errors/systemerror"));
const debug = require('debug')('bind:rest:decorators');
const TAG = 'MAKE-PARAMS-EXTRACTOR';
function makeParamExtractorFactory(t, paramName) {
    debug('%s Entered makeParamExtractorFactory with t="%s", paramName="%s"', TAG, controllerparamtype_1.default[t], paramName);
    let ret;
    switch (t) {
        case controllerparamtype_1.default.Cookie:
            ret = (c) => (ctx) => {
                return ctx.parsedCookies[paramName];
            };
            break;
        case controllerparamtype_1.default.ContextParam:
            ret = (c) => (ctx) => {
                return ctx.storage[paramName];
            };
            break;
        case controllerparamtype_1.default.QueryParam:
            ret = (c) => (ctx) => {
                return ctx.parsedUrlQuery[paramName];
            };
            break;
        case controllerparamtype_1.default.Header:
            ret = (c) => (ctx) => {
                return ctx.requestHeaders[paramName];
            };
            break;
        case controllerparamtype_1.default.PathParam:
            ret = (c) => (ctx) => {
                var _a, _b;
                const pathParam = (_b = (_a = ctx.routeParams) === null || _a === void 0 ? void 0 : _a.pathParams) === null || _b === void 0 ? void 0 : _b.find((p) => p.paramName === paramName);
                return pathParam === null || pathParam === void 0 ? void 0 : pathParam.paramValue;
            };
            break;
        default:
            throw new systemerror_1.default(`Unsupported ControllerParamType type ${controllerparamtype_1.default[t]}`);
    }
    return ret;
}
exports.default = makeParamExtractorFactory;
//# sourceMappingURL=makeparamextractorfactory.js.map