"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BodyParam = exports.RequestMethod = exports.ContextStore = exports.Context = exports.BindContainer = exports.Request = exports.Router = exports.Headers = exports.ParsedQuery = exports.Cookies = exports.QueryString = exports.Body = exports.Required = void 0;
const holiday_router_1 = require("holiday-router");
const bind_di_1 = require("bind-di");
const controllerparamtype_1 = __importDefault(require("../../enums/controllerparamtype"));
const metaprops_1 = require("../metaprops");
const controllermethodparams_1 = require("../../consts/controllermethodparams");
const parsebody_1 = require("../../utils/parsebody");
const makeparamdecorator_1 = __importDefault(require("./makeparamdecorator"));
const applysingledecorator_1 = __importDefault(require("./applysingledecorator"));
const getparamtype_1 = __importDefault(require("./getparamtype"));
const badrequest_1 = __importDefault(require("../../errors/http/badrequest"));
const headernames_1 = __importDefault(require("../../consts/headernames"));
const debug = require('debug')('bind:rest:decorators');
const TAG = 'NO_ARG_METHOD_DECORATOR';
function Required(target, propertyKey, parameterIndex) {
    return applysingledecorator_1.default(target, propertyKey, parameterIndex, true);
}
exports.Required = Required;
function Body(target, propertyKey, parameterIndex) {
    const paramTypes = Reflect.getMetadata(bind_di_1.PARAM_TYPES, target, propertyKey);
    const controllerName = `${target.constructor.name}.${propertyKey}`;
    const paramName = bind_di_1.getMethodParamName(target, propertyKey, parameterIndex);
    const paramType = getparamtype_1.default(paramTypes, parameterIndex);
    if (paramType === controllermethodparams_1.PARAM_TYPE_PROMISE) {
        throw new Error(`Invalid argument type ${controllerName} 
    paramName="${paramName}" (argument ${parameterIndex})
        @Body param cannot be of type Promise.`);
    }
    const primitiveType = paramType === controllermethodparams_1.PARAM_TYPE_STRING ||
        paramType === controllermethodparams_1.PARAM_TYPE_BOOLEAN ||
        paramType === controllermethodparams_1.PARAM_TYPE_NUMBER;
    const shouldJsonParseBody = !primitiveType && paramType !== controllermethodparams_1.PARAM_TYPE_OBJECT;
    debug('%s Body decorator. propertyKey=%s parameterInded=%s primitiveType=%s shouldJsonParseBody=%s', TAG, propertyKey, parameterIndex, primitiveType, shouldJsonParseBody);
    const paramFactory = (c) => {
        return function BodyExtractor(context) {
            var _a, _b;
            let jsonSchema;
            if (shouldJsonParseBody) {
                try {
                    jsonSchema = Reflect.getMetadata(metaprops_1.SYM_JSON_SCHEMA, paramType);
                }
                catch (e) {
                    debug('%s exception from getMetadata of paramType=%o error %o', TAG, paramType, e);
                }
                debug('%s jsonSchema=%o', TAG, jsonSchema);
            }
            debug('%s trying to get request content-type', TAG);
            const contentType = ((_b = (_a = context.requestHeaders) === null || _a === void 0 ? void 0 : _a[headernames_1.default.CONTENT_TYPE]) === null || _b === void 0 ? void 0 : _b.toLowerCase()) || '';
            debug('%s in Body parser. contentType=%s', TAG, contentType);
            let parsed;
            if ((!primitiveType && contentType.startsWith(controllermethodparams_1.CONTENT_TYPE_JSON)) || jsonSchema) {
                debug('%s will parseJsonBody', TAG);
                parsed = parsebody_1.parseJsonBody(context, jsonSchema);
            }
            else {
                debug('%s will parse plain body', TAG);
                parsed = parsebody_1.parseBody(context);
            }
            return parsed.catch((err) => {
                debug('%s ERROR parsing body %o', TAG, err);
                throw new badrequest_1.default(`Failed to parse request body.
        Controller="${controllerName}" 
        Parameter="${paramName}" (argument ${parameterIndex})
        Error=${err.message}
        `, err);
            });
        };
    };
    return applysingledecorator_1.default(target, propertyKey, parameterIndex, false, controllerparamtype_1.default.Body, paramFactory);
}
exports.Body = Body;
exports.QueryString = makeparamdecorator_1.default((c) => (context) => context.querystring);
exports.Cookies = makeparamdecorator_1.default((c) => (context) => context.parsedCookies);
exports.ParsedQuery = makeparamdecorator_1.default((c) => (context) => context.parsedUrlQuery);
exports.Headers = makeparamdecorator_1.default((c) => (context) => context.requestHeaders);
exports.Router = makeparamdecorator_1.default((c) => {
    const componentDetails = c.getComponentDetails(bind_di_1.Identity(holiday_router_1.HttpRouter));
    return (context) => {
        return componentDetails.get([context]);
    };
});
exports.Request = makeparamdecorator_1.default((c) => (context) => context.req);
exports.BindContainer = makeparamdecorator_1.default((c) => (context) => c);
exports.Context = makeparamdecorator_1.default((c) => (context) => context);
exports.ContextStore = makeparamdecorator_1.default((c) => (context) => context.storage);
exports.RequestMethod = makeparamdecorator_1.default((c) => (context) => context.requestMethod);
exports.BodyParam = makeparamdecorator_1.default();
//# sourceMappingURL=noargmethodparams.js.map