"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const bind_di_1 = require("bind-di");
const metaprops_1 = require("../metaprops");
const applicationerror_1 = __importDefault(require("../../errors/applicationerror"));
const debug = require('debug')('bind:rest:decorators');
const TAG = '@Path';
const decorateControllerClass = (target, path) => {
    const ctrlPath = Reflect.getMetadata(metaprops_1.SYM_REQUEST_PATH, target);
    if (ctrlPath) {
        debug('%s "%s" already defined on controller "%s" Attempted to set another value="%s"', TAG, ctrlPath, target.name, path);
    }
    if (path && path[0] !== '/') {
        throw new applicationerror_1.default(`Annotation @Path '${path}' of controller class is invalid. It must start with a '/' Did you mean /${path}'?`);
    }
    Reflect.defineMetadata(metaprops_1.SYM_REQUEST_PATH, path, target);
    return undefined;
};
const decorateControllerMethod = (target, propertyKey, descriptor, path) => {
    let normalizedPath = path;
    if (!propertyKey) {
        throw new applicationerror_1.default(`method name is not provided to Path decorator 
    for class=${target.constructor.name}`);
    }
    if (!descriptor || !descriptor.value || typeof descriptor.value !== 'function') {
        throw new applicationerror_1.default(`Cannot apply @Path to "${target.constructor.name}.${propertyKey}"
    because property "${propertyKey}" is not a function`);
    }
    debug('Defining %s path="%s" for controller="%s.%s"', TAG, path, target.constructor.name, propertyKey);
    if (path[0] !== '/') {
        normalizedPath = `/${path}`;
    }
    Reflect.defineMetadata(metaprops_1.SYM_REQUEST_PATH, normalizedPath, target, propertyKey);
    return undefined;
};
function Path(path) {
    debug('Path decorator called with %s', path);
    return function pathDecorator(target, propertyKey, descriptor) {
        debug('%s entered pathDecorator', TAG);
        const targetType = bind_di_1.getTargetStereotype(target);
        let res;
        switch (targetType) {
            case bind_di_1.TargetStereoType.CONSTRUCTOR:
                res = decorateControllerClass(target, path);
                break;
            case bind_di_1.TargetStereoType.PROTOTYPE:
                res = decorateControllerMethod(target, propertyKey, descriptor, path);
                break;
            default:
                throw new applicationerror_1.default(`@Path decorator cannot be applied to this type of target`);
        }
        return res;
    };
}
exports.default = Path;
//# sourceMappingURL=pathdecorator.js.map