"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Validate = exports.toAsyncValidator = void 0;
const bind_di_1 = require("bind-di");
const metaprops_1 = require("../metaprops");
const consts_1 = require("../../consts");
const toAsyncValidator = (validator) => {
    return (c) => (ctx) => validator;
};
exports.toAsyncValidator = toAsyncValidator;
function Validate(...validators) {
    return (target, propertyKey, index) => {
        var _a;
        const paramName = bind_di_1.getMethodParamName(target, propertyKey, index);
        const validatorFunc = function paramValidator(container) {
            return function paramValidator2(ctx) {
                return function paramValidator3(param) {
                    const validationResults = validators
                        .map((f) => f(container))
                        .map((f) => f(ctx))
                        .map((f) => f(param));
                    return Promise.all(validationResults)
                        .then((results) => {
                        return results.reduce((acc, next) => {
                            if (bind_di_1.isDefined(next)) {
                                acc.push(next.message);
                            }
                            return acc;
                        }, []);
                    })
                        .then((errors) => {
                        if (errors.length > 0) {
                            return new Error(`Parameter "${paramName}" (argument ${index})
                Errors:\n${errors.join(consts_1.DOTTED_LINE)}
                `);
                        }
                        return undefined;
                    })
                        .catch((e) => {
                        return new Error(`Parameter "${paramName}" (argument ${index})
              Error: ${e.message}
                `);
                    });
                };
            };
        };
        const metaDetails = Reflect.getMetadata(metaprops_1.SYM_METHOD_PARAMS, target, propertyKey) || [];
        if (!metaDetails[index]) {
            metaDetails[index] = {
                f: undefined,
                isRequired: false,
                paramName: '',
                paramDecoratorType: undefined,
            };
        }
        if (metaDetails[index] && metaDetails[index].validator) {
            throw new Error(`Method parameter validator for argument ${index} already defined 
      on method ${(_a = target.constructor) === null || _a === void 0 ? void 0 : _a.name}.${propertyKey}`);
        }
        metaDetails[index].validator = validatorFunc;
        Reflect.defineMetadata(metaprops_1.SYM_METHOD_PARAMS, metaDetails, target, propertyKey);
    };
}
exports.Validate = Validate;
//# sourceMappingURL=validate.js.map