"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TRACE = exports.PATCH = exports.OPTIONS = exports.HEAD = exports.DELETE = exports.PUT = exports.POST = exports.GET = exports.decorate = void 0;
const bind_di_1 = require("bind-di");
const http_method_enum_1 = __importDefault(require("http-method-enum"));
const metaprops_1 = require("../metaprops");
const pathdecorator_1 = __importDefault(require("../controller/pathdecorator"));
const debug = require('debug')('bind:rest:decorators');
const TAG = 'ADD_METHOD_ANNOTATION';
function addMethodAnnotation(target, propertyKey, method, descriptor) {
    let p = Reflect.getMetadata(metaprops_1.SYM_REQUEST_METHOD, target, propertyKey);
    const rt = Reflect.getMetadata(bind_di_1.RETURN_TYPE, target, propertyKey);
    debug(`Adding Method annotation %s to ${target.constructor.name}.${propertyKey} 
    returnType: ${rt}
    with descriptor=%s`, method, descriptor);
    p = bind_di_1.getOrElse(p, new Set());
    p.add(method);
    debug('%s Annotated methods %o', TAG, p);
    Reflect.defineMetadata(metaprops_1.SYM_REQUEST_METHOD, p, target, propertyKey);
    return undefined;
}
const decorate = (pathOrTarget, propertyKey, method, descriptor) => {
    if (typeof pathOrTarget === 'string') {
        return (target, prop, desc) => {
            addMethodAnnotation(target, prop, method, desc);
            pathdecorator_1.default(pathOrTarget)(target, prop, desc);
        };
    }
    return addMethodAnnotation(pathOrTarget, propertyKey, method, descriptor);
};
exports.decorate = decorate;
function GET(target, propertyKey, descriptor) {
    return exports.decorate(target, propertyKey, http_method_enum_1.default.GET, descriptor);
}
exports.GET = GET;
function POST(target, propertyKey, descriptor) {
    return exports.decorate(target, propertyKey, http_method_enum_1.default.POST, descriptor);
}
exports.POST = POST;
function PUT(target, propertyKey, descriptor) {
    return exports.decorate(target, propertyKey, http_method_enum_1.default.PUT, descriptor);
}
exports.PUT = PUT;
function DELETE(target, propertyKey, descriptor) {
    return exports.decorate(target, propertyKey, http_method_enum_1.default.DELETE, descriptor);
}
exports.DELETE = DELETE;
function HEAD(target, propertyKey, descriptor) {
    return exports.decorate(target, propertyKey, http_method_enum_1.default.HEAD, descriptor);
}
exports.HEAD = HEAD;
function OPTIONS(target, propertyKey, descriptor) {
    return exports.decorate(target, propertyKey, http_method_enum_1.default.OPTIONS, descriptor);
}
exports.OPTIONS = OPTIONS;
function PATCH(target, propertyKey, descriptor) {
    return exports.decorate(target, propertyKey, http_method_enum_1.default.PATCH, descriptor);
}
exports.PATCH = PATCH;
function TRACE(target, propertyKey, descriptor) {
    return exports.decorate(target, propertyKey, http_method_enum_1.default.TRACE, descriptor);
}
exports.TRACE = TRACE;
//# sourceMappingURL=requestmethods.js.map