"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AfterResponse = exports.Afterware = exports.Middleware = exports.decorateMiddleware = void 0;
require("reflect-metadata");
const bind_di_1 = require("bind-di");
const metaprops_1 = require("./metaprops");
const debug = require('debug')('bind:rest:decorators');
const TAG = 'Middleware';
const validatePriority = (i) => {
    const res = Number(i);
    return !Number.isNaN(res) && res >= 0 && res <= 1000;
};
function decorateMiddleware(target, priority, decoratorName = 'Middleware') {
    debug('Defining %s for constructor %s. decoratorName=%s', TAG, target.name, decoratorName);
    bind_di_1.Component(target);
    bind_di_1.defineMetadata(bind_di_1.DEFAULT_SCOPE, bind_di_1.ComponentScope.NEWINSTANCE, target)();
    const metaData = Reflect.getMetadata(bind_di_1.COMPONENT_META_DATA, target) || {};
    metaData[metaprops_1.MIDDLEWARE_PRIORITY] = priority;
    bind_di_1.defineMetadata(bind_di_1.COMPONENT_META_DATA, metaData, target)();
}
exports.decorateMiddleware = decorateMiddleware;
function Middleware(val) {
    if (typeof val === 'number') {
        if (!validatePriority(val)) {
            throw new TypeError(`Value passed to @Middleware decorator '${val}' is invalid.  
      It must be a number between 0 and 1000`);
        }
        return function middlewareDecorator(constructor) {
            decorateMiddleware(constructor, Number.MIN_SAFE_INTEGER + val, 'Middleware');
        };
    }
    decorateMiddleware(val, -1);
}
exports.Middleware = Middleware;
function Afterware(priority) {
    if (!validatePriority(priority)) {
        throw new TypeError(`Value passed to @Middleware decorator '${priority}' is invalid.  
      It must be a number between 0 and 1000`);
    }
    return function middlewareDecorator(constructor) {
        decorateMiddleware(constructor, priority, 'Afterware');
    };
}
exports.Afterware = Afterware;
function AfterResponse(priority) {
    return function middlewareDecorator(constructor) {
        decorateMiddleware(constructor, priority + 1000, 'AfterResponse');
    };
}
exports.AfterResponse = AfterResponse;
//# sourceMappingURL=middleware.js.map