"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Middlewares = exports.toMWFuncFactory = exports.toMWFactory = void 0;
require("reflect-metadata");
const bind_di_1 = require("bind-di");
const metaprops_1 = require("./metaprops");
const debug = require('debug')('bind:rest:decorators');
const TAG = '@Middlewares';
const toMWFactory = (middleware) => {
    return (container) => {
        const mwID = Reflect.getMetadata(bind_di_1.COMPONENT_IDENTITY, middleware);
        debug('%s got mwID="%s"', TAG, mwID);
        const componentDetails = container.getComponentDetails(mwID);
        return (context) => {
            const oMW = componentDetails.get([context]);
            return oMW.doFilter(context);
        };
    };
};
exports.toMWFactory = toMWFactory;
const toMWFuncFactory = (arr) => {
    return (container) => {
        const aMW = arr.map((f) => f(container));
        return (context) => {
            return aMW.reduce((acc, next) => {
                return acc.then(next);
            }, Promise.resolve(context));
        };
    };
};
exports.toMWFuncFactory = toMWFuncFactory;
function Middlewares(...middlewares) {
    return function middlewaresDecorator(target, propertyKey, descriptor) {
        var _a;
        const targetStereotype = bind_di_1.getTargetStereotype(target);
        switch (targetStereotype) {
            case bind_di_1.TargetStereoType.CONSTRUCTOR:
                break;
            case bind_di_1.TargetStereoType.PROTOTYPE:
                if (!propertyKey ||
                    !descriptor ||
                    !descriptor.value ||
                    typeof descriptor.value !== 'function') {
                    throw new Error(`@Middlewares decorator can be applied only to Class 
          or controller method.
          It was applied to unsupported property of class "${(_a = target === null || target === void 0 ? void 0 : target.constructor) === null || _a === void 0 ? void 0 : _a.name}"`);
                }
                break;
            default:
                throw new Error(`@Middlewares decorator can be applied only to Class 
        or controller method.`);
        }
        let aMiddlewares = Reflect.getMetadata(metaprops_1.SYM_CONTROLLER_MIDDLEWARES, target, propertyKey) || [];
        aMiddlewares = aMiddlewares.concat(middlewares.map(exports.toMWFactory));
        debug(`Adding ${TAG} decorator with ${aMiddlewares.length} middlewares 
    to '${target.constructor.name}.${propertyKey}' controller.`);
        Reflect.defineMetadata(metaprops_1.SYM_CONTROLLER_MIDDLEWARES, aMiddlewares, target, propertyKey);
        const extraDependencies = middlewares.map((dep) => {
            const ret = Reflect.getMetadata(bind_di_1.COMPONENT_IDENTITY, dep);
            if (!ret) {
                throw new Error(`${TAG} decorator. Could not determine identity of middleware ${dep.name}`);
            }
            return ret;
        });
        Reflect.defineMetadata(bind_di_1.EXTRA_DEPENDENCIES, extraDependencies, target, propertyKey);
        if (targetStereotype === bind_di_1.TargetStereoType.PROTOTYPE) {
            Reflect.defineMetadata(bind_di_1.EXTRA_DEPENDENCIES, extraDependencies, target.constructor, propertyKey);
        }
    };
}
exports.Middlewares = Middlewares;
//# sourceMappingURL=middlewares.js.map