"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getHTTPOverHTTPTunnel = exports.getHTTPSOverHTTPTunnel = void 0;
const tunnel_1 = __importDefault(require("tunnel"));
const debug = require('debug')('bind:rest:httprequest');
const TAG = 'TUNNEL';
function getHTTPSOverHTTPTunnel(options) {
    debug('%s ENTERED getHTTPSOverHTTPTunnel with ', TAG, options);
    const proxyOptions = {
        maxSockets: options.poolSize || 10,
        proxy: {
            host: options.proxy.proxyHost,
            port: options.proxy.proxyPort,
            headers: options.proxy.headers || { 'User-Agent': 'Promise-Of-T Browser' },
        },
    };
    if (options.ca) {
        proxyOptions.ca = options.ca;
    }
    if (options.key) {
        proxyOptions.key = options.key;
    }
    if (options.cert) {
        proxyOptions.cert = options.cert;
    }
    if (options.proxy.auth) {
        debug(`Setting proxyAuth on tunnel to ${options.proxy.auth}`);
        proxyOptions.proxy.proxyAuth = options.proxy.auth;
    }
    proxyOptions.rejectUnauthorized = options.requestOptions.rejectUnauthorized;
    debug('%s HTTPS proxy options: ', TAG, proxyOptions);
    try {
        const ret = tunnel_1.default.httpsOverHttp(proxyOptions);
        debug('Returning tunnel object');
        return ret;
    }
    catch (ex) {
        debug('http tunnel exception. ex.message=%s', ex.message);
        throw new Error('Failed to create https tunnel object');
    }
}
exports.getHTTPSOverHTTPTunnel = getHTTPSOverHTTPTunnel;
function getHTTPOverHTTPTunnel(options) {
    debug('ENTERED getHTTPOverHTTPTunnel with proxyOptions ', options);
    const proxyOptions = {
        maxSockets: options.poolSize || 10,
        proxy: {
            host: options.proxy.proxyHost,
            port: options.proxy.proxyPort,
            headers: options.proxy.headers || { 'User-Agent': 'Promise-Of-T Browser' },
        },
    };
    if (options.proxy.auth) {
        proxyOptions.proxy.proxyAuth = options.proxy.auth;
    }
    const tunnelingAgent = tunnel_1.default.httpOverHttp(proxyOptions);
    return tunnelingAgent;
}
exports.getHTTPOverHTTPTunnel = getHTTPOverHTTPTunnel;
//# sourceMappingURL=tunnel.js.map