"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const charset_1 = __importDefault(require("charset"));
const supportedencodings_1 = __importDefault(require("../consts/supportedencodings"));
const interfaces_1 = require("../interfaces");
const core_1 = require("../core");
const maxhttpbodysize_1 = __importDefault(require("../consts/maxhttpbodysize"));
const headernames_1 = __importDefault(require("../consts/headernames"));
const debug = require('debug')('bind:rest:httprequest');
const TAG = 'STRINGIFY_BODY';
const stringifyBody = (maxBytes = maxhttpbodysize_1.default) => (resp) => {
    if (interfaces_1.isAppResponseWithBody(resp)) {
        debug('%s response already has body %s', TAG, resp.body);
        return Promise.resolve(resp);
    }
    const is = resp.getReadStream();
    let bytesRead = 0;
    const bufs = [];
    let cs = 'utf-8';
    if (resp.headers && resp.headers[headernames_1.default.CONTENT_TYPE]) {
        debug('Have content-type header in response: %s', resp.headers[headernames_1.default.CONTENT_TYPE]);
        cs = charset_1.default(resp.headers[headernames_1.default.CONTENT_TYPE]);
        cs = cs && cs.toLocaleLowerCase();
        debug(`Charset from response: %s`, cs);
        if (cs === 'iso-8859-1' || cs === 'iso8859-1' || cs === 'latin-1' || cs === 'iso88591') {
            debug('Changed charset to latin1');
            cs = 'latin1';
        }
    }
    if (cs) {
        if (!supportedencodings_1.default.includes(cs)) {
            debug('Unknown encoding: ', cs);
        }
        else {
        }
    }
    return new Promise(function stringifyBodyResolver(resolve, reject) {
        is.on('data', function stringifyBodyOnData(data) {
            bytesRead += data.byteLength;
            if (bytesRead > maxBytes) {
                const maxSizeError = `Body is larger than maximum allowed size of ${maxBytes} bytes`;
                debug('%s %s', TAG, maxSizeError);
                if (is.destroy && typeof is.destroy === 'function') {
                    debug('%s calling destroy() method', TAG);
                    is.destroy(maxSizeError);
                }
                else {
                    debug('%s read stream does not have destroy() function.', TAG);
                    reject(new Error(maxSizeError));
                    is.removeAllListeners();
                }
            }
            else {
                bufs.push(data);
            }
        });
        is.on('end', function stringifyBodyOnEnd() {
            debug('%s onEnd called. Total chunks=%s', TAG, bufs.length);
            const finalBuffer = Buffer.concat(bufs);
            debug('%s finalBuffer bytes=%s  Calling toString with encoding=%s', TAG, finalBuffer.byteLength, cs);
            const str = finalBuffer.toString(cs);
            resolve(new core_1.AppResponse(str, resp.statusCode, resp.headers));
        });
        is.on('error', function stringifyBodyOnError(err) {
            reject(err);
        });
    });
};
exports.default = stringifyBody;
//# sourceMappingURL=stringifybody.js.map