"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const http_1 = __importDefault(require("http"));
const core_1 = require("../core");
const debug = require('debug')('bind:rest:runtime:application');
const DEFAULT_PORT = 9099;
function createServer(config) {
    const app = new core_1.Application(config);
    const APPLICATION_PORT = process.env.BIND_REST_HTTP_PORT && Number.parseInt(process.env.BIND_REST_HTTP_PORT, 10);
    const PORT = !APPLICATION_PORT ? DEFAULT_PORT : APPLICATION_PORT;
    debug(`STARTING APPLICATION on PORT=${PORT}. Please wait while application initializes`);
    return app
        .init()
        .then(() => {
        debug('Application initialized');
        debug(app.toString());
        http_1.default.createServer(app.handleRequest.bind(app)).listen(PORT);
        const message = `Application "${app.toString()}" running on port ${PORT}`;
        debug(message);
        return message;
    })
        .catch((e) => {
        console.error(`Application initialization failed ${e.message} ${e.stack}`);
        process.exit(1);
    });
}
exports.default = createServer;
//# sourceMappingURL=createserver.js.map