"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseJsonBody = exports.parseBody = void 0;
const inflation_1 = __importDefault(require("inflation"));
const raw_body_1 = __importDefault(require("raw-body"));
const jsonschema_1 = require("jsonschema");
const schemavalidationerror_1 = __importDefault(require("../errors/validation/schemavalidationerror"));
const debug = require('debug')('bind:rest:runtime:parseBody');
const TAG = 'PARSE-BODY';
function parseBody(ctx) {
    const allowedMethods = ['PUT', 'POST'];
    if (!allowedMethods.includes(ctx.requestMethod)) {
        debug('%s bad request method for parseBody %s', TAG, ctx.requestMethod);
        throw new Error(`
  Error: Cannot extract Body from Request.
  Reason: request method "${ctx.requestMethod}" cannot include request body`);
    }
    if (ctx.requestBody) {
        debug('%s context already has parsedBody, returning it', TAG);
        return Promise.resolve(ctx.requestBody);
    }
    const ret = raw_body_1.default(inflation_1.default(ctx.req))
        .then((rawBody) => {
        const res = String(rawBody);
        debug('%s returning rawBody as String %s', TAG, res);
        const parsedBody = res.valueOf();
        ctx.requestBody = parsedBody;
        return parsedBody;
    })
        .catch((e) => {
        debug('%s ERROR parsing body %o', TAG, e);
        return Promise.reject(e);
    });
    return ret;
}
exports.parseBody = parseBody;
function parseJsonBody(ctx, schema) {
    debug('%s Entered parseJsonBody', TAG);
    if (ctx.parsedBody) {
        debug('%s returning cached parsedBody %o', TAG, ctx.parsedBody);
        return Promise.resolve(ctx.parsedBody);
    }
    return parseBody(ctx)
        .then((body) => {
        const ret = JSON.parse(body);
        debug('%s parsed body as json %o', TAG, ret);
        return ret;
    })
        .then((obj) => {
        let validator;
        if (schema) {
            debug('%s have schema to validate', TAG);
            validator = new jsonschema_1.Validator();
            const res = validator.validate(obj, schema, { propertyName: 'Object' });
            if (!res.valid) {
                debug('%s Failed to validate schema', TAG);
                throw new schemavalidationerror_1.default(`Schema Validation Error="${res.toString()}"`);
            }
        }
        Reflect.set(ctx, 'parsedBody', obj);
        return obj;
    });
}
exports.parseJsonBody = parseJsonBody;
//# sourceMappingURL=parsebody.js.map