"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CehLogger = exports.stringifyHeaders = exports.stringifyMessage = exports.formatTimeStamp = exports.truncateMessage = exports.LOG_LEVEL = void 0;
const bind_di_1 = require("bind-di");
const settings_1 = require("./settings");
const HDR_TRANSACTION_ID = 'sm_transactionid';
const HDR_ADPSAMANAME = 'adpsamaname';
const HDR_ADPAID = 'adpaid';
const HDR_AZURE_USER = 'azure_user';
const HDR_ADPEMAIL = 'adpemail';
const HDR_CEHEMAIL = 'ceh_email';
const DEFAULT_TRANSACTION_ID = 'NO-TRANSACTION-ID';
const DEFAULT_ADPSAMANAME = 'NO-ADPSAMANAME';
const DEFAULT_ADPAID = 'NO-ADPAID';
const HDR_EMPTY_VALUE = '-';
var LOG_LEVEL;
(function (LOG_LEVEL) {
    LOG_LEVEL[LOG_LEVEL["DEBUG"] = 0] = "DEBUG";
    LOG_LEVEL[LOG_LEVEL["INFO"] = 1] = "INFO";
    LOG_LEVEL[LOG_LEVEL["WARN"] = 2] = "WARN";
    LOG_LEVEL[LOG_LEVEL["ERROR"] = 3] = "ERROR";
    LOG_LEVEL[LOG_LEVEL["FATAL"] = 4] = "FATAL";
})(LOG_LEVEL = exports.LOG_LEVEL || (exports.LOG_LEVEL = {}));
const DEFAULT_LOG_LEVEL = LOG_LEVEL.INFO;
const toLogLevelName = (level) => {
    return LOG_LEVEL[level];
};
const truncateMessage = (message, limit) => {
    if (message.length <= limit) {
        return message;
    }
    return `${message.substring(0, limit)} LOG_TRUNCATED...`;
};
exports.truncateMessage = truncateMessage;
const formatTimeStamp = (dt) => {
    let ret = dt.toISOString()
        .replace('000Z', dt.getMilliseconds() + 'Z');
    return ret;
};
exports.formatTimeStamp = formatTimeStamp;
const stringifyMessage = (message) => {
    const mType = typeof message;
    if (message === undefined) {
        return '-UNDEFINED-';
    }
    let res;
    switch (mType) {
        case 'string':
            if (message === '') {
                res = 'EMPTY-STRING';
            }
            else {
                res = message;
            }
            break;
        case 'number':
            res = String(message);
            break;
        case 'function':
            res = `FUNCTION ${message.toString()}`;
            break;
        case 'object':
            try {
                res = JSON.stringify(message);
            }
            catch (e) {
                res = ` BAD-JSON ${e.message}`;
            }
            break;
        default:
            res = `LogMetaType="${mType}"`;
    }
    return res;
};
exports.stringifyMessage = stringifyMessage;
const stringifyHeaders = (headers) => {
    const transactionId = (headers === null || headers === void 0 ? void 0 : headers[HDR_TRANSACTION_ID]) || DEFAULT_TRANSACTION_ID;
    const adpsamaname = (headers === null || headers === void 0 ? void 0 : headers[HDR_ADPSAMANAME]) || DEFAULT_ADPSAMANAME;
    const adpaid = (headers === null || headers === void 0 ? void 0 : headers[HDR_ADPAID]) || DEFAULT_ADPAID;
    const azure_user = (headers === null || headers === void 0 ? void 0 : headers[HDR_AZURE_USER]) || HDR_EMPTY_VALUE;
    const adpemail = (headers === null || headers === void 0 ? void 0 : headers[HDR_ADPEMAIL]) || HDR_EMPTY_VALUE;
    const ceh_email = (headers === null || headers === void 0 ? void 0 : headers[HDR_CEHEMAIL]) || HDR_EMPTY_VALUE;
    return `sm_transactionid=${transactionId} adpsamaname=${adpsamaname} adpaid=${adpaid} azure_user="${azure_user}" ceh_email="${ceh_email}" azure_email="${adpemail}"`;
};
exports.stringifyHeaders = stringifyHeaders;
let CehLogger = class CehLogger {
    constructor(settings) {
        this.logLevel = LOG_LEVEL[settings.LOG_LEVEL.toUpperCase()] || DEFAULT_LOG_LEVEL;
        this.logLimit = settings.LOG_LIMIT || Number.MAX_SAFE_INTEGER;
    }
    getWriteStreams(_logLevel) {
        return [process.stdout];
    }
    format(logLevel, messages) {
        var _a;
        let headers;
        let tail = '';
        let message;
        let messageLimit;
        const head = `EventTime=${exports.formatTimeStamp(new Date())} CehLogLevel=${toLogLevelName(logLevel)}`;
        if ((_a = messages[messages.length - 1]) === null || _a === void 0 ? void 0 : _a.sm_transactionid) {
            headers = messages.pop();
            tail = exports.stringifyHeaders(headers);
        }
        messageLimit = this.logLimit - (head.length + tail.length);
        message = messages.reduce((acc, next) => {
            if (acc.length <= messageLimit) {
                return exports.truncateMessage(`${acc} ${exports.stringifyMessage(next)}`, messageLimit);
            }
            return acc;
        }, head);
        return `${message} ${tail}`;
    }
    doLog(logLevel, messages) {
        let ws;
        let logMessage;
        if (logLevel >= this.logLevel) {
            logMessage = this.format(logLevel, messages);
            ws = this.getWriteStreams(logLevel);
            ws.forEach(stream => stream.write(logMessage));
        }
        return undefined;
    }
    fatal(...messages) {
        this.doLog(LOG_LEVEL.FATAL, messages);
    }
    error(...messages) {
        this.doLog(LOG_LEVEL.ERROR, messages);
    }
    warn(...messages) {
        this.doLog(LOG_LEVEL.WARN, messages);
    }
    info(...messages) {
        this.doLog(LOG_LEVEL.INFO, messages);
    }
    debug(...messages) {
        this.doLog(LOG_LEVEL.DEBUG, messages);
    }
};
CehLogger = __decorate([
    bind_di_1.Component,
    __param(0, bind_di_1.Inject),
    __metadata("design:paramtypes", [settings_1.Settings])
], CehLogger);
exports.CehLogger = CehLogger;
//# sourceMappingURL=cehlogger.js.map