/// <reference types="node" />
import { UrlWithStringQuery } from 'url';
import * as http from 'http';
import { ParsedUrlQuery } from 'querystring';
import { ComponentScope, ComponentIdentity, Maybe } from 'bind-di';
import { IUriParams } from 'holiday-router';
import HttpStatusCode from 'http-status-enum';
import HTTPMethod from 'http-method-enum';
import { IAppResponse } from '../lib/interfaces/appresponse';
import { IStoredComponent } from '../lib/interfaces/storedcomponent';
import { IContextStore } from '../lib/types/contextstore';
import { IBindRestContext } from '../lib/interfaces/icontext';
import { IResponseCookieValue } from '../lib/interfaces/responsecookie';
export default class BindRestContext implements IBindRestContext {
    static readonly id: ComponentIdentity;
    contextType: string;
    protected readonly responseCookies: NodeJS.Dict<IResponseCookieValue>;
    static readonly create: (req: http.IncomingMessage) => BindRestContext;
    protected httpRequestBody: string;
    protected httpRequest?: http.IncomingMessage;
    protected httpRequestMethod: HTTPMethod;
    protected reqUrl: string;
    protected uriInfo: UrlWithStringQuery;
    protected cookies: NodeJS.Dict<string>;
    protected query: ParsedUrlQuery;
    protected myControllerName: string;
    protected myRouteParams: IUriParams;
    protected responseHeaders: NodeJS.Dict<string>;
    protected responseStatusCode: HttpStatusCode;
    protected requestStartTime: number;
    protected response: IAppResponse;
    setHeader(key: string, value: string): void;
    setStatusCode(statusCode: HttpStatusCode): void;
    get req(): http.IncomingMessage;
    set req(req: http.IncomingMessage);
    get requestHeaders(): http.IncomingHttpHeaders;
    get requestBody(): string | undefined;
    set requestBody(body: string);
    get requestMethod(): HTTPMethod;
    set appResponse(response: IAppResponse);
    get appResponse(): Maybe<IAppResponse>;
    protected scopedComponents: Array<IStoredComponent>;
    readonly storage: IContextStore;
    readonly errors: Array<Error>;
    readonly scope = ComponentScope.REQUEST;
    get startTime(): number;
    getComponent(id: ComponentIdentity): Maybe<Object>;
    setComponent(identity: ComponentIdentity, component: any): void;
    get controllerName(): string;
    get routeParams(): IUriParams;
    set routeParams(params: IUriParams);
    set controllerName(name: string);
    get path(): string;
    get requestUrl(): string;
    private parseUrl;
    get querystring(): string;
    get parsedUrlQuery(): ParsedUrlQuery;
    get parsedCookies(): NodeJS.Dict<string>;
    controllerArguments: Array<any>;
    parsedBody: any;
    getResponseCookies(): NodeJS.Dict<IResponseCookieValue>;
    setResponseCookie(name: string, value: IResponseCookieValue): void;
}
